// required for old g++ to compile PRId64 macros, see
// https://github.com/pytorch/pytorch/issues/3571
// for context
#ifndef __STDC_FORMAT_MACROS
#define __STDC_FORMAT_MACROS
#endif

// NOTE: This condition is true for all PyTorch internal libraries, it
//       just excludes external projects such as torch_xla which
//       re-use some of the PyTorch codegen machinery.
#if defined(CAFFE2_BUILD_MAIN_LIB)        || \
    defined(TORCH_CUDA_BUILD_MAIN_LIB)    || \
    defined(TORCH_HIP_BUILD_MAIN_LIB)     || \
    defined(TORCH_CUDA_CU_BUILD_MAIN_LIB) || \
    defined(TORCH_CUDA_CPP_BUILD_MAIN_LIB)
#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
#endif

// @generated by tools/codegen/gen.py from RegisterDispatchKey.cpp

#include <c10/core/TensorImpl.h>
#include <c10/core/Allocator.h>
#include <ATen/DeviceGuard.h>
#include <ATen/NamedTensorUtils.h>
#include <ATen/Utils.h>
#include <ATen/WrapDimUtils.h>
#include <ATen/Dispatch.h>
#include <c10/util/ExclusivelyOwned.h>
#include <c10/util/Half.h>
#include <c10/core/UndefinedTensorImpl.h>
#include <c10/util/Optional.h>
#include <ATen/Tensor.h>
#include <ATen/native/Resize.h>

#include <cstddef>
#include <functional>
#include <memory>
#include <utility>

#include <ATen/Config.h>
#include <ATen/core/op_registration/adaption.h>
#include <torch/library.h>


#include <ATen/ops/as_strided_native.h>
#include <ATen/ops/empty.h>
#include <ATen/ops/empty_strided.h>
#include <ATen/ops/_conj_physical_native.h>
#include <ATen/ops/_nnz_native.h>
#include <ATen/ops/abs_native.h>
#include <ATen/ops/add_native.h>
#include <ATen/ops/addmm_native.h>
#include <ATen/ops/addmv_native.h>
#include <ATen/ops/angle_native.h>
#include <ATen/ops/asin_native.h>
#include <ATen/ops/asinh_native.h>
#include <ATen/ops/atan_native.h>
#include <ATen/ops/atanh_native.h>
#include <ATen/ops/ceil_native.h>
#include <ATen/ops/clone_native.h>
#include <ATen/ops/col_indices_native.h>
#include <ATen/ops/conj_physical_native.h>
#include <ATen/ops/copy_native.h>
#include <ATen/ops/crow_indices_native.h>
#include <ATen/ops/empty_like_native.h>
#include <ATen/ops/empty_native.h>
#include <ATen/ops/erf_native.h>
#include <ATen/ops/erfinv_native.h>
#include <ATen/ops/expm1_native.h>
#include <ATen/ops/floor_native.h>
#include <ATen/ops/isinf_native.h>
#include <ATen/ops/isnan_native.h>
#include <ATen/ops/isneginf_native.h>
#include <ATen/ops/isposinf_native.h>
#include <ATen/ops/log1p_native.h>
#include <ATen/ops/mm_native.h>
#include <ATen/ops/neg_native.h>
#include <ATen/ops/normal_native.h>
#include <ATen/ops/rad2deg_native.h>
#include <ATen/ops/resize_as_sparse_native.h>
#include <ATen/ops/resize_native.h>
#include <ATen/ops/round_native.h>
#include <ATen/ops/sgn_native.h>
#include <ATen/ops/sign_native.h>
#include <ATen/ops/signbit_native.h>
#include <ATen/ops/sin_native.h>
#include <ATen/ops/sinh_native.h>
#include <ATen/ops/sqrt_native.h>
#include <ATen/ops/tan_native.h>
#include <ATen/ops/tanh_native.h>
#include <ATen/ops/to_dense_native.h>
#include <ATen/ops/triangular_solve_native.h>
#include <ATen/ops/trunc_native.h>
#include <ATen/ops/values_native.h>


namespace at {

// NB: TORCH_LIBRARY_IMPL must be in an anonymous namespace to avoid
// ambiguity with conflicting identifiers that may have been defined in
// at namespace already.
namespace {


void resize_out(const Tensor &out, IntArrayRef sizes, IntArrayRef strides, const TensorOptions &options) {
  TORCH_CHECK(options.dtype() == out.dtype(),
      "Expected out tensor to have dtype ", options.dtype(), ", but got ", out.dtype(), " instead");
  TORCH_CHECK(options.device() == out.device(),
      "Expected out tensor to have device ", options.device(), ", but got ", out.device(), " instead");
  const bool resized = at::native::resize_output(out, sizes);
  // Only restride if a resize occurred; otherwise we ignore the (advisory)
  // strides from the meta function and directly use the output tensor's
  // preexisting strides
  if (resized) {
    if (!strides.empty()) {
      TORCH_INTERNAL_ASSERT(!options.memory_format_opt().has_value());
      at::native::as_strided_(out, sizes, strides);
    } else if (options.memory_format_opt().has_value()) {
      out.unsafeGetTensorImpl()->empty_tensor_restride(*options.memory_format_opt());
    }
  }
}

void check_inplace(const Tensor &self, IntArrayRef sizes, const TensorOptions &options) {
  // These checks are needed on those operators that:
  //   1) don't use 'TensorIterator' (e.g. 'addmm' and 'baddbmm')
  //   2) have particular typing rules (e.g. 'cumsum' and 'cumprod')
  // For other operators (e.g. 'add'), 'TensorIterator' already checks
  // these things separately.
  TORCH_CHECK(options.dtype() == self.dtype(),
      "Bad in-place call: ",
      "input tensor dtype ", self.dtype(), " and output tensor dtype ", options.dtype(), " should match");
  TORCH_CHECK(options.device() == self.device(),
      "Bad in-place call: ",
      "input tensor device ", self.device(), " and output tensor device ", options.device(), " should match");
  TORCH_CHECK(sizes == self.sizes(),
      "Bad in-place call: ",
      "input tensor size ", self.sizes(), " and output tensor size ", sizes, " should match");
}

namespace {

at::Tensor wrapper__abs(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::abs_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_abs_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::abs_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__abs_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::abs_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__angle(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::angle_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_angle_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::angle_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__sgn(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sgn_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_sgn_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::sgn_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__sgn_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sgn_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper___conj_physical(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::conj_physical_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_conj_physical_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::conj_physical_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__conj_physical_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::conj_physical_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper_Tensor_add_Tensor(const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
    // No device check


  // DeviceGuard omitted
  return at::native::add_sparse_csr(self, other, alpha);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_add_out_out(const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::add_out_sparse_csr_cpu(self, other, alpha, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_Tensor_add__Tensor(at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
    // No device check


  // DeviceGuard omitted
  return at::native::add_sparse_csr_(self, other, alpha);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_addmv_out_out(const at::Tensor & self, const at::Tensor & mat, const at::Tensor & vec, const at::Scalar & beta, const at::Scalar & alpha, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::addmv_out_sparse_csr(self, mat, vec, beta, alpha, out);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__asinh(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::asinh_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_asinh_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::asinh_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__asinh_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::asinh_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__atanh(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::atanh_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_atanh_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::atanh_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__atanh_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::atanh_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__asin(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::asin_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_asin_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::asin_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__asin_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::asin_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__atan(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::atan_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_atan_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::atan_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__atan_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::atan_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__ceil(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::ceil_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_ceil_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::ceil_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__ceil_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::ceil_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__copy_(at::Tensor & self, const at::Tensor & src, bool non_blocking) {
    // No device check


  // DeviceGuard omitted
  return at::native::copy_sparse_csr_(self, src, non_blocking);
}

} // anonymous namespace
namespace {

at::Tensor wrapper_memory_format_empty_memory_format(at::IntArrayRef size, c10::optional<at::ScalarType> dtype, c10::optional<at::Layout> layout, c10::optional<at::Device> device, c10::optional<bool> pin_memory, c10::optional<at::MemoryFormat> memory_format) {
    // No device check


  // DeviceGuard omitted
  return at::native::empty_sparse_csr(size, dtype, layout, device, pin_memory, memory_format);
}

} // anonymous namespace
namespace {

const at::Tensor & wrapper__resize_(const at::Tensor & self, at::IntArrayRef size, c10::optional<at::MemoryFormat> memory_format) {
    // No device check


  // DeviceGuard omitted
  return at::native::resize_sparse_csr_(self, size, memory_format);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__empty_like(const at::Tensor & self, c10::optional<at::ScalarType> dtype, c10::optional<at::Layout> layout, c10::optional<at::Device> device, c10::optional<bool> pin_memory, c10::optional<at::MemoryFormat> memory_format) {
    // No device check


  // DeviceGuard omitted
  return at::native::empty_like_sparse_csr(self, dtype, layout, device, pin_memory, memory_format);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__erf(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::erf_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_erf_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::erf_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__erf_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::erf_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__expm1(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::expm1_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_expm1_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::expm1_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__expm1_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::expm1_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__floor(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::floor_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_floor_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::floor_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__floor_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::floor_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__isnan(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::isnan_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__log1p(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::log1p_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_log1p_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::log1p_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__log1p_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::log1p_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__mm(const at::Tensor & self, const at::Tensor & mat2) {
    // No device check


  // DeviceGuard omitted
  return at::native::_sparse_csr_mm(self, mat2);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_mm_out_out(const at::Tensor & self, const at::Tensor & mat2, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::_sparse_csr_mm_out(self, mat2, out);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__rad2deg(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::rad2deg_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_rad2deg_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::rad2deg_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__rad2deg_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::rad2deg_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__neg(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::neg_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_neg_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::neg_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__neg_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::neg_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__round(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::round_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_round_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::round_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__round_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::round_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__sin(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sin_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_sin_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::sin_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__sin_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sin_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__sinh(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sinh_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_sinh_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::sinh_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__sinh_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sinh_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__sqrt(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sqrt_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_sqrt_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::sqrt_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__sqrt_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sqrt_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__tan(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::tan_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_tan_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::tan_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__tan_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::tan_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__tanh(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::tanh_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_tanh_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::tanh_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__tanh_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::tanh_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__trunc(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::trunc_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_trunc_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::trunc_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__trunc_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::trunc_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__clone(const at::Tensor & self, c10::optional<at::MemoryFormat> memory_format) {
    // No device check


  // DeviceGuard omitted
  return at::native::clone_sparse_csr(self, memory_format);
}

} // anonymous namespace
namespace {

const at::Tensor & wrapper__resize_as_sparse_(const at::Tensor & self, const at::Tensor & the_template) {
    // No device check


  // DeviceGuard omitted
  return at::native::resize_as_sparse_csr_(self, the_template);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__addmm(const at::Tensor & self, const at::Tensor & mat1, const at::Tensor & mat2, const at::Scalar & beta, const at::Scalar & alpha) {
    // No device check


  // DeviceGuard omitted
  return at::native::addmm_sparse_csr_dense(self, mat1, mat2, beta, alpha);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_addmm_out_out(const at::Tensor & self, const at::Tensor & mat1, const at::Tensor & mat2, const at::Scalar & beta, const at::Scalar & alpha, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::addmm_out_sparse_csr_cpu(self, mat1, mat2, beta, alpha, out);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__to_dense(const at::Tensor & self, c10::optional<at::ScalarType> dtype) {
    // No device check


  // DeviceGuard omitted
  return at::native::sparse_to_dense(self, dtype);
}

} // anonymous namespace
namespace {

int64_t wrapper___nnz(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::_nnz_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__values(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::values_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__crow_indices(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::crow_indices_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__col_indices(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::col_indices_sparse_csr(self);
}

} // anonymous namespace
namespace {

::std::tuple<at::Tensor &,at::Tensor &> wrapper_X_triangular_solve_out_X(const at::Tensor & self, const at::Tensor & A, bool upper, bool transpose, bool unitriangular, at::Tensor & X, at::Tensor & M) {
    // No device check


  // DeviceGuard omitted
  return at::native::triangular_solve_out_sparse_csr_cpu(self, A, upper, transpose, unitriangular, X, M);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__erfinv(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::erfinv_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_erfinv_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::erfinv_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__erfinv_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::erfinv_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__sign(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sign_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_sign_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::sign_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__sign_(at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::sign_sparse_csr_(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__signbit(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::signbit_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_signbit_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::signbit_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper__normal_(at::Tensor & self, double mean, double std, c10::optional<at::Generator> generator) {
    // No device check


  // DeviceGuard omitted
  return at::native::normal_sparse_csr_(self, mean, std, generator);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__isinf(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::isinf_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__isposinf(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::isposinf_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_isposinf_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::isposinf_sparse_csr_out(self, out);
}

} // anonymous namespace
namespace {

at::Tensor wrapper__isneginf(const at::Tensor & self) {
    // No device check


  // DeviceGuard omitted
  return at::native::isneginf_sparse_csr(self);
}

} // anonymous namespace
namespace {

at::Tensor & wrapper_out_isneginf_out_out(const at::Tensor & self, at::Tensor & out) {
    // No device check


  // DeviceGuard omitted
  return at::native::isneginf_sparse_csr_out(self, out);
}

} // anonymous namespace

TORCH_LIBRARY_IMPL(aten, SparseCsrCPU, m) {
  m.impl("abs",
  TORCH_FN(wrapper__abs));
  m.impl("abs.out",
  TORCH_FN(wrapper_out_abs_out_out));
  m.impl("abs_",
  TORCH_FN(wrapper__abs_));
  m.impl("angle",
  TORCH_FN(wrapper__angle));
  m.impl("angle.out",
  TORCH_FN(wrapper_out_angle_out_out));
  m.impl("sgn",
  TORCH_FN(wrapper__sgn));
  m.impl("sgn.out",
  TORCH_FN(wrapper_out_sgn_out_out));
  m.impl("sgn_",
  TORCH_FN(wrapper__sgn_));
  m.impl("_conj_physical",
  TORCH_FN(wrapper___conj_physical));
  m.impl("conj_physical.out",
  TORCH_FN(wrapper_out_conj_physical_out_out));
  m.impl("conj_physical_",
  TORCH_FN(wrapper__conj_physical_));
  m.impl("add.Tensor",
  TORCH_FN(wrapper_Tensor_add_Tensor));
  m.impl("add.out",
  TORCH_FN(wrapper_out_add_out_out));
  m.impl("add_.Tensor",
  TORCH_FN(wrapper_Tensor_add__Tensor));
  m.impl("addmv.out",
  TORCH_FN(wrapper_out_addmv_out_out));
  m.impl("asinh",
  TORCH_FN(wrapper__asinh));
  m.impl("asinh.out",
  TORCH_FN(wrapper_out_asinh_out_out));
  m.impl("asinh_",
  TORCH_FN(wrapper__asinh_));
  m.impl("atanh",
  TORCH_FN(wrapper__atanh));
  m.impl("atanh.out",
  TORCH_FN(wrapper_out_atanh_out_out));
  m.impl("atanh_",
  TORCH_FN(wrapper__atanh_));
  m.impl("asin",
  TORCH_FN(wrapper__asin));
  m.impl("asin.out",
  TORCH_FN(wrapper_out_asin_out_out));
  m.impl("asin_",
  TORCH_FN(wrapper__asin_));
  m.impl("atan",
  TORCH_FN(wrapper__atan));
  m.impl("atan.out",
  TORCH_FN(wrapper_out_atan_out_out));
  m.impl("atan_",
  TORCH_FN(wrapper__atan_));
  m.impl("ceil",
  TORCH_FN(wrapper__ceil));
  m.impl("ceil.out",
  TORCH_FN(wrapper_out_ceil_out_out));
  m.impl("ceil_",
  TORCH_FN(wrapper__ceil_));
  m.impl("copy_",
  TORCH_FN(wrapper__copy_));
  m.impl("empty.memory_format",
  TORCH_FN(wrapper_memory_format_empty_memory_format));
  m.impl("resize_",
  TORCH_FN(wrapper__resize_));
  m.impl("empty_like",
  TORCH_FN(wrapper__empty_like));
  m.impl("erf",
  TORCH_FN(wrapper__erf));
  m.impl("erf.out",
  TORCH_FN(wrapper_out_erf_out_out));
  m.impl("erf_",
  TORCH_FN(wrapper__erf_));
  m.impl("expm1",
  TORCH_FN(wrapper__expm1));
  m.impl("expm1.out",
  TORCH_FN(wrapper_out_expm1_out_out));
  m.impl("expm1_",
  TORCH_FN(wrapper__expm1_));
  m.impl("floor",
  TORCH_FN(wrapper__floor));
  m.impl("floor.out",
  TORCH_FN(wrapper_out_floor_out_out));
  m.impl("floor_",
  TORCH_FN(wrapper__floor_));
  m.impl("isnan",
  TORCH_FN(wrapper__isnan));
  m.impl("log1p",
  TORCH_FN(wrapper__log1p));
  m.impl("log1p.out",
  TORCH_FN(wrapper_out_log1p_out_out));
  m.impl("log1p_",
  TORCH_FN(wrapper__log1p_));
  m.impl("mm",
  TORCH_FN(wrapper__mm));
  m.impl("mm.out",
  TORCH_FN(wrapper_out_mm_out_out));
  m.impl("rad2deg",
  TORCH_FN(wrapper__rad2deg));
  m.impl("rad2deg.out",
  TORCH_FN(wrapper_out_rad2deg_out_out));
  m.impl("rad2deg_",
  TORCH_FN(wrapper__rad2deg_));
  m.impl("neg",
  TORCH_FN(wrapper__neg));
  m.impl("neg.out",
  TORCH_FN(wrapper_out_neg_out_out));
  m.impl("neg_",
  TORCH_FN(wrapper__neg_));
  m.impl("round",
  TORCH_FN(wrapper__round));
  m.impl("round.out",
  TORCH_FN(wrapper_out_round_out_out));
  m.impl("round_",
  TORCH_FN(wrapper__round_));
  m.impl("sin",
  TORCH_FN(wrapper__sin));
  m.impl("sin.out",
  TORCH_FN(wrapper_out_sin_out_out));
  m.impl("sin_",
  TORCH_FN(wrapper__sin_));
  m.impl("sinh",
  TORCH_FN(wrapper__sinh));
  m.impl("sinh.out",
  TORCH_FN(wrapper_out_sinh_out_out));
  m.impl("sinh_",
  TORCH_FN(wrapper__sinh_));
  m.impl("sqrt",
  TORCH_FN(wrapper__sqrt));
  m.impl("sqrt.out",
  TORCH_FN(wrapper_out_sqrt_out_out));
  m.impl("sqrt_",
  TORCH_FN(wrapper__sqrt_));
  m.impl("tan",
  TORCH_FN(wrapper__tan));
  m.impl("tan.out",
  TORCH_FN(wrapper_out_tan_out_out));
  m.impl("tan_",
  TORCH_FN(wrapper__tan_));
  m.impl("tanh",
  TORCH_FN(wrapper__tanh));
  m.impl("tanh.out",
  TORCH_FN(wrapper_out_tanh_out_out));
  m.impl("tanh_",
  TORCH_FN(wrapper__tanh_));
  m.impl("trunc",
  TORCH_FN(wrapper__trunc));
  m.impl("trunc.out",
  TORCH_FN(wrapper_out_trunc_out_out));
  m.impl("trunc_",
  TORCH_FN(wrapper__trunc_));
  m.impl("clone",
  TORCH_FN(wrapper__clone));
  m.impl("resize_as_sparse_",
  TORCH_FN(wrapper__resize_as_sparse_));
  m.impl("addmm",
  TORCH_FN(wrapper__addmm));
  m.impl("addmm.out",
  TORCH_FN(wrapper_out_addmm_out_out));
  m.impl("to_dense",
  TORCH_FN(wrapper__to_dense));
  m.impl("_nnz",
  TORCH_FN(wrapper___nnz));
  m.impl("values",
  TORCH_FN(wrapper__values));
  m.impl("crow_indices",
  TORCH_FN(wrapper__crow_indices));
  m.impl("col_indices",
  TORCH_FN(wrapper__col_indices));
  m.impl("triangular_solve.X",
  TORCH_FN(wrapper_X_triangular_solve_out_X));
  m.impl("erfinv",
  TORCH_FN(wrapper__erfinv));
  m.impl("erfinv.out",
  TORCH_FN(wrapper_out_erfinv_out_out));
  m.impl("erfinv_",
  TORCH_FN(wrapper__erfinv_));
  m.impl("sign",
  TORCH_FN(wrapper__sign));
  m.impl("sign.out",
  TORCH_FN(wrapper_out_sign_out_out));
  m.impl("sign_",
  TORCH_FN(wrapper__sign_));
  m.impl("signbit",
  TORCH_FN(wrapper__signbit));
  m.impl("signbit.out",
  TORCH_FN(wrapper_out_signbit_out_out));
  m.impl("normal_",
  TORCH_FN(wrapper__normal_));
  m.impl("isinf",
  TORCH_FN(wrapper__isinf));
  m.impl("isposinf",
  TORCH_FN(wrapper__isposinf));
  m.impl("isposinf.out",
  TORCH_FN(wrapper_out_isposinf_out_out));
  m.impl("isneginf",
  TORCH_FN(wrapper__isneginf));
  m.impl("isneginf.out",
  TORCH_FN(wrapper_out_isneginf_out_out));
}

} // anonymous namespace

namespace sparsecsrcpu {


at::Tensor abs(const at::Tensor & self) {
return wrapper__abs(self);
}

at::Tensor & abs_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_abs_out_out(self, out);
}

at::Tensor & abs_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_abs_out_out(self, out);
}

at::Tensor & abs_(at::Tensor & self) {
return wrapper__abs_(self);
}

at::Tensor angle(const at::Tensor & self) {
return wrapper__angle(self);
}

at::Tensor & angle_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_angle_out_out(self, out);
}

at::Tensor & angle_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_angle_out_out(self, out);
}

at::Tensor sgn(const at::Tensor & self) {
return wrapper__sgn(self);
}

at::Tensor & sgn_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_sgn_out_out(self, out);
}

at::Tensor & sgn_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_sgn_out_out(self, out);
}

at::Tensor & sgn_(at::Tensor & self) {
return wrapper__sgn_(self);
}

at::Tensor _conj_physical(const at::Tensor & self) {
return wrapper___conj_physical(self);
}

at::Tensor & conj_physical_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_conj_physical_out_out(self, out);
}

at::Tensor & conj_physical_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_conj_physical_out_out(self, out);
}

at::Tensor & conj_physical_(at::Tensor & self) {
return wrapper__conj_physical_(self);
}

at::Tensor add(const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
return wrapper_Tensor_add_Tensor(self, other, alpha);
}

at::Tensor & add_out(at::Tensor & out, const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
return wrapper_out_add_out_out(self, other, alpha, out);
}

at::Tensor & add_outf(const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha, at::Tensor & out) {
return wrapper_out_add_out_out(self, other, alpha, out);
}

at::Tensor & add_(at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
return wrapper_Tensor_add__Tensor(self, other, alpha);
}

at::Tensor & addmv_out(at::Tensor & out, const at::Tensor & self, const at::Tensor & mat, const at::Tensor & vec, const at::Scalar & beta, const at::Scalar & alpha) {
return wrapper_out_addmv_out_out(self, mat, vec, beta, alpha, out);
}

at::Tensor & addmv_outf(const at::Tensor & self, const at::Tensor & mat, const at::Tensor & vec, const at::Scalar & beta, const at::Scalar & alpha, at::Tensor & out) {
return wrapper_out_addmv_out_out(self, mat, vec, beta, alpha, out);
}

at::Tensor asinh(const at::Tensor & self) {
return wrapper__asinh(self);
}

at::Tensor & asinh_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_asinh_out_out(self, out);
}

at::Tensor & asinh_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_asinh_out_out(self, out);
}

at::Tensor & asinh_(at::Tensor & self) {
return wrapper__asinh_(self);
}

at::Tensor atanh(const at::Tensor & self) {
return wrapper__atanh(self);
}

at::Tensor & atanh_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_atanh_out_out(self, out);
}

at::Tensor & atanh_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_atanh_out_out(self, out);
}

at::Tensor & atanh_(at::Tensor & self) {
return wrapper__atanh_(self);
}

at::Tensor asin(const at::Tensor & self) {
return wrapper__asin(self);
}

at::Tensor & asin_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_asin_out_out(self, out);
}

at::Tensor & asin_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_asin_out_out(self, out);
}

at::Tensor & asin_(at::Tensor & self) {
return wrapper__asin_(self);
}

at::Tensor atan(const at::Tensor & self) {
return wrapper__atan(self);
}

at::Tensor & atan_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_atan_out_out(self, out);
}

at::Tensor & atan_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_atan_out_out(self, out);
}

at::Tensor & atan_(at::Tensor & self) {
return wrapper__atan_(self);
}

at::Tensor ceil(const at::Tensor & self) {
return wrapper__ceil(self);
}

at::Tensor & ceil_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_ceil_out_out(self, out);
}

at::Tensor & ceil_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_ceil_out_out(self, out);
}

at::Tensor & ceil_(at::Tensor & self) {
return wrapper__ceil_(self);
}

at::Tensor & copy_(at::Tensor & self, const at::Tensor & src, bool non_blocking) {
return wrapper__copy_(self, src, non_blocking);
}

at::Tensor empty(at::IntArrayRef size, at::TensorOptions options, c10::optional<at::MemoryFormat> memory_format) {
return wrapper_memory_format_empty_memory_format(size, optTypeMetaToScalarType(options.dtype_opt()), options.layout_opt(), options.device_opt(), options.pinned_memory_opt(), c10::impl::check_tensor_options_and_extract_memory_format(options, memory_format));
}

at::Tensor empty(at::IntArrayRef size, c10::optional<at::ScalarType> dtype, c10::optional<at::Layout> layout, c10::optional<at::Device> device, c10::optional<bool> pin_memory, c10::optional<at::MemoryFormat> memory_format) {
return wrapper_memory_format_empty_memory_format(size, dtype, layout, device, pin_memory, memory_format);
}

const at::Tensor & resize_(const at::Tensor & self, at::IntArrayRef size, c10::optional<at::MemoryFormat> memory_format) {
return wrapper__resize_(self, size, memory_format);
}

at::Tensor empty_like(const at::Tensor & self, at::TensorOptions options, c10::optional<at::MemoryFormat> memory_format) {
return wrapper__empty_like(self, optTypeMetaToScalarType(options.dtype_opt()), options.layout_opt(), options.device_opt(), options.pinned_memory_opt(), c10::impl::check_tensor_options_and_extract_memory_format(options, memory_format));
}

at::Tensor empty_like(const at::Tensor & self, c10::optional<at::ScalarType> dtype, c10::optional<at::Layout> layout, c10::optional<at::Device> device, c10::optional<bool> pin_memory, c10::optional<at::MemoryFormat> memory_format) {
return wrapper__empty_like(self, dtype, layout, device, pin_memory, memory_format);
}

at::Tensor erf(const at::Tensor & self) {
return wrapper__erf(self);
}

at::Tensor & erf_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_erf_out_out(self, out);
}

at::Tensor & erf_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_erf_out_out(self, out);
}

at::Tensor & erf_(at::Tensor & self) {
return wrapper__erf_(self);
}

at::Tensor expm1(const at::Tensor & self) {
return wrapper__expm1(self);
}

at::Tensor & expm1_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_expm1_out_out(self, out);
}

at::Tensor & expm1_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_expm1_out_out(self, out);
}

at::Tensor & expm1_(at::Tensor & self) {
return wrapper__expm1_(self);
}

at::Tensor floor(const at::Tensor & self) {
return wrapper__floor(self);
}

at::Tensor & floor_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_floor_out_out(self, out);
}

at::Tensor & floor_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_floor_out_out(self, out);
}

at::Tensor & floor_(at::Tensor & self) {
return wrapper__floor_(self);
}

at::Tensor isnan(const at::Tensor & self) {
return wrapper__isnan(self);
}

at::Tensor log1p(const at::Tensor & self) {
return wrapper__log1p(self);
}

at::Tensor & log1p_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_log1p_out_out(self, out);
}

at::Tensor & log1p_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_log1p_out_out(self, out);
}

at::Tensor & log1p_(at::Tensor & self) {
return wrapper__log1p_(self);
}

at::Tensor mm(const at::Tensor & self, const at::Tensor & mat2) {
return wrapper__mm(self, mat2);
}

at::Tensor & mm_out(at::Tensor & out, const at::Tensor & self, const at::Tensor & mat2) {
return wrapper_out_mm_out_out(self, mat2, out);
}

at::Tensor & mm_outf(const at::Tensor & self, const at::Tensor & mat2, at::Tensor & out) {
return wrapper_out_mm_out_out(self, mat2, out);
}

at::Tensor rad2deg(const at::Tensor & self) {
return wrapper__rad2deg(self);
}

at::Tensor & rad2deg_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_rad2deg_out_out(self, out);
}

at::Tensor & rad2deg_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_rad2deg_out_out(self, out);
}

at::Tensor & rad2deg_(at::Tensor & self) {
return wrapper__rad2deg_(self);
}

at::Tensor neg(const at::Tensor & self) {
return wrapper__neg(self);
}

at::Tensor & neg_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_neg_out_out(self, out);
}

at::Tensor & neg_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_neg_out_out(self, out);
}

at::Tensor & neg_(at::Tensor & self) {
return wrapper__neg_(self);
}

at::Tensor round(const at::Tensor & self) {
return wrapper__round(self);
}

at::Tensor & round_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_round_out_out(self, out);
}

at::Tensor & round_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_round_out_out(self, out);
}

at::Tensor & round_(at::Tensor & self) {
return wrapper__round_(self);
}

at::Tensor sin(const at::Tensor & self) {
return wrapper__sin(self);
}

at::Tensor & sin_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_sin_out_out(self, out);
}

at::Tensor & sin_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_sin_out_out(self, out);
}

at::Tensor & sin_(at::Tensor & self) {
return wrapper__sin_(self);
}

at::Tensor sinh(const at::Tensor & self) {
return wrapper__sinh(self);
}

at::Tensor & sinh_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_sinh_out_out(self, out);
}

at::Tensor & sinh_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_sinh_out_out(self, out);
}

at::Tensor & sinh_(at::Tensor & self) {
return wrapper__sinh_(self);
}

at::Tensor sqrt(const at::Tensor & self) {
return wrapper__sqrt(self);
}

at::Tensor & sqrt_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_sqrt_out_out(self, out);
}

at::Tensor & sqrt_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_sqrt_out_out(self, out);
}

at::Tensor & sqrt_(at::Tensor & self) {
return wrapper__sqrt_(self);
}

at::Tensor tan(const at::Tensor & self) {
return wrapper__tan(self);
}

at::Tensor & tan_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_tan_out_out(self, out);
}

at::Tensor & tan_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_tan_out_out(self, out);
}

at::Tensor & tan_(at::Tensor & self) {
return wrapper__tan_(self);
}

at::Tensor tanh(const at::Tensor & self) {
return wrapper__tanh(self);
}

at::Tensor & tanh_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_tanh_out_out(self, out);
}

at::Tensor & tanh_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_tanh_out_out(self, out);
}

at::Tensor & tanh_(at::Tensor & self) {
return wrapper__tanh_(self);
}

at::Tensor trunc(const at::Tensor & self) {
return wrapper__trunc(self);
}

at::Tensor & trunc_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_trunc_out_out(self, out);
}

at::Tensor & trunc_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_trunc_out_out(self, out);
}

at::Tensor & trunc_(at::Tensor & self) {
return wrapper__trunc_(self);
}

at::Tensor clone(const at::Tensor & self, c10::optional<at::MemoryFormat> memory_format) {
return wrapper__clone(self, memory_format);
}

const at::Tensor & resize_as_sparse_(const at::Tensor & self, const at::Tensor & the_template) {
return wrapper__resize_as_sparse_(self, the_template);
}

at::Tensor addmm(const at::Tensor & self, const at::Tensor & mat1, const at::Tensor & mat2, const at::Scalar & beta, const at::Scalar & alpha) {
return wrapper__addmm(self, mat1, mat2, beta, alpha);
}

at::Tensor & addmm_out(at::Tensor & out, const at::Tensor & self, const at::Tensor & mat1, const at::Tensor & mat2, const at::Scalar & beta, const at::Scalar & alpha) {
return wrapper_out_addmm_out_out(self, mat1, mat2, beta, alpha, out);
}

at::Tensor & addmm_outf(const at::Tensor & self, const at::Tensor & mat1, const at::Tensor & mat2, const at::Scalar & beta, const at::Scalar & alpha, at::Tensor & out) {
return wrapper_out_addmm_out_out(self, mat1, mat2, beta, alpha, out);
}

at::Tensor to_dense(const at::Tensor & self, c10::optional<at::ScalarType> dtype) {
return wrapper__to_dense(self, dtype);
}

int64_t _nnz(const at::Tensor & self) {
return wrapper___nnz(self);
}

at::Tensor values(const at::Tensor & self) {
return wrapper__values(self);
}

at::Tensor crow_indices(const at::Tensor & self) {
return wrapper__crow_indices(self);
}

at::Tensor col_indices(const at::Tensor & self) {
return wrapper__col_indices(self);
}

::std::tuple<at::Tensor &,at::Tensor &> triangular_solve_out(at::Tensor & X, at::Tensor & M, const at::Tensor & self, const at::Tensor & A, bool upper, bool transpose, bool unitriangular) {
return wrapper_X_triangular_solve_out_X(self, A, upper, transpose, unitriangular, X, M);
}

::std::tuple<at::Tensor &,at::Tensor &> triangular_solve_outf(const at::Tensor & self, const at::Tensor & A, bool upper, bool transpose, bool unitriangular, at::Tensor & X, at::Tensor & M) {
return wrapper_X_triangular_solve_out_X(self, A, upper, transpose, unitriangular, X, M);
}

at::Tensor erfinv(const at::Tensor & self) {
return wrapper__erfinv(self);
}

at::Tensor & erfinv_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_erfinv_out_out(self, out);
}

at::Tensor & erfinv_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_erfinv_out_out(self, out);
}

at::Tensor & erfinv_(at::Tensor & self) {
return wrapper__erfinv_(self);
}

at::Tensor sign(const at::Tensor & self) {
return wrapper__sign(self);
}

at::Tensor & sign_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_sign_out_out(self, out);
}

at::Tensor & sign_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_sign_out_out(self, out);
}

at::Tensor & sign_(at::Tensor & self) {
return wrapper__sign_(self);
}

at::Tensor signbit(const at::Tensor & self) {
return wrapper__signbit(self);
}

at::Tensor & signbit_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_signbit_out_out(self, out);
}

at::Tensor & signbit_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_signbit_out_out(self, out);
}

at::Tensor & normal_(at::Tensor & self, double mean, double std, c10::optional<at::Generator> generator) {
return wrapper__normal_(self, mean, std, generator);
}

at::Tensor isinf(const at::Tensor & self) {
return wrapper__isinf(self);
}

at::Tensor isposinf(const at::Tensor & self) {
return wrapper__isposinf(self);
}

at::Tensor & isposinf_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_isposinf_out_out(self, out);
}

at::Tensor & isposinf_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_isposinf_out_out(self, out);
}

at::Tensor isneginf(const at::Tensor & self) {
return wrapper__isneginf(self);
}

at::Tensor & isneginf_out(at::Tensor & out, const at::Tensor & self) {
return wrapper_out_isneginf_out_out(self, out);
}

at::Tensor & isneginf_outf(const at::Tensor & self, at::Tensor & out) {
return wrapper_out_isneginf_out_out(self, out);
}

} // namespace sparsecsrcpu

} // namespace at
